#include "AProtonGun.hh"

#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4ThreeVector.hh"
#include "G4ParticleMomentum.hh"



//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

AProtonGun::AProtonGun(G4double beamStartingPosition) 
{
  // number of particles per event
  G4int n_particle = 1;
  particleGun = new G4ParticleGun(n_particle);

  // get the table of particles defined in GEANT4
  particleTable = G4ParticleTable::GetParticleTable();

  // find the particle to simulate
  particle = particleTable->FindParticle("proton");

  // define a gun with the particle to simulate
  particleGun->SetParticleDefinition(particle);

  // set the energy of the particle
  particleGun->SetParticleEnergy(100.*CLHEP::MeV);

  // set the initial position of the particle
  particleGun->SetParticlePosition(G4ThreeVector(0., 0., beamStartingPosition));

  // set the direction of the particle
  particleGun->SetParticleMomentumDirection(G4ThreeVector(0,0,1.));


  // To print some info
  G4cout << G4endl << " ------ INFO ------ Your are working with a Proton Gun " << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

AProtonGun::~AProtonGun()
{
  delete particleGun;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void AProtonGun::GeneratePrimaries(G4Event* anEvent)
{
  // generate a primary vertex -> start a new particle
  particleGun->GeneratePrimaryVertex(anEvent); 

  return;
}
