//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
// $Id$
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "MyGun.hh"
#include "MyGunMessenger.hh"

#include "G4RunManager.hh"
#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4SystemOfUnits.hh"

#include <cmath>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

MyGun::MyGun(G4double b)
{
  G4int n_particle = 1;
  particleGun  = new G4ParticleGun(n_particle);
  
  //create a messenger for this class
  gunMessenger = new MyGunMessenger(this);


  beamStartingPosition = b;
  
  // default particle kinematic
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();

  particleName = "proton"; // default particle -> you can change it with the messenger class
  particle = particleTable->FindParticle(particleName);
  particleGun->SetParticleDefinition(particle);
  particleGun->SetParticleMomentumDirection(G4ThreeVector(0.,0.,1.));
  particleGun->SetParticleEnergy(50.*MeV);
  particleGun->SetParticlePosition(G4ThreeVector(0.,0.,beamStartingPosition));


  // To print some info
  G4cout << G4endl << " ------ INFO ------ Your are working with the particle gun that has a messenger class" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

MyGun::~MyGun()
{
  delete particleGun;
  delete gunMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void MyGun::GeneratePrimaries(G4Event* anEvent)
{ 
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  particle = particleTable->FindParticle(particleName);
  particleGun->SetParticleDefinition(particle);

  // isotropic source
  G4double cosTheta = 2.*G4UniformRand() - 1.;
  G4double phi = twopi*G4UniformRand();
  
  G4double sinTheta = std::sqrt(1. - cosTheta*cosTheta);

  G4double ux = sinTheta*std::cos(phi);
  G4double uy = sinTheta*std::sin(phi);
  G4double uz = cosTheta;

  particleGun->SetParticleMomentumDirection(G4ThreeVector(ux,uy,uz));
  particleGun->SetParticlePosition(G4ThreeVector(0.*cm, 0.*cm, beamStartingPosition));


  particleGun->GeneratePrimaryVertex(anEvent);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

