#include "MyGunReadingFile.hh"

#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4ThreeVector.hh"
#include "G4ParticleMomentum.hh"

#include "G4RunManager.hh"

#include <sstream>
#include <string>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

MyGunReadingFile::MyGunReadingFile(G4double beamStartingPosition) 
{
  // number of particles per event
  G4int n_particle = 1;
  particleGun = new G4ParticleGun(n_particle);

  // get the table of particles defined in GEANT4
  particleTable = G4ParticleTable::GetParticleTable();

  // find the particle to simulate
  particle = particleTable->FindParticle("gamma");

  // define a gun with the particle to simulate
  particleGun->SetParticleDefinition(particle);

  // set the energy of the particle
  particleGun->SetParticleEnergy(10.*CLHEP::MeV);

  // set the initial position of the particle
  particleGun->SetParticlePosition(G4ThreeVector(0., 0., beamStartingPosition));

  // set the direction of the particle
  particleGun->SetParticleMomentumDirection(G4ThreeVector(0,0,1.));



  file = new std::ifstream("particle_generator.txt");

  if( !file->good() ) G4Exception("MyGunReadingFile::MyGunReadingFile()",
				     "file = new ifstream(\"particle_generator.txt\");",
				     FatalException,
				     "the particle generator file was not opened");
  
  std::string line("");
  std::getline(*file, line); // discard first line

  // To print some info
  G4cout << G4endl << " ------ INFO ------ Your are working with the particle gun reading the external generator " << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

MyGunReadingFile::~MyGunReadingFile()
{
  delete particleGun;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void MyGunReadingFile::GeneratePrimaries(G4Event* anEvent)
{
  
  std::string line("");
  std::getline(*file, line);

  if(file->eof()){
    // the file has ended -> do something for this event
    G4RunManager::GetRunManager()->AbortRun(true);
    anEvent->SetEventAborted();
    
    // dummy particle only to end this event
    particle = particleTable->FindParticle("geantino");
    particleGun->SetParticleDefinition(particle);
    particleGun->SetParticleEnergy(1.*CLHEP::MeV);
    particleGun->SetParticlePosition(G4ThreeVector(0.*CLHEP::cm,0.*CLHEP::cm,0.*CLHEP::cm));
    particleGun->SetParticleMomentumDirection(G4ThreeVector(0,0,-1.));
  }
  else{
    // the file still has particles to be simulated
    G4double x;
    G4double y;
    G4double z;
    G4double dx;
    G4double dy;
    G4double dz;

    // with a stringstream the conversion from string to G4double is automatic
    std::istringstream buffer( line );
    buffer >> x >> y >> z >> dx >> dy >> dz; 

    particleGun->SetParticlePosition(G4ThreeVector(x*CLHEP::cm, y*CLHEP::cm, z*CLHEP::cm));
    particleGun->SetParticleMomentumDirection(G4ThreeVector(dx, dy, dz));
  }


  // generate a primary vertex -> start a new particle
  particleGun->GeneratePrimaryVertex(anEvent); 

  return;
}
